@@ -35,6 +35,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
+#include <glib/gstdio.h>
 #include <locale.h>
 #ifdef HAVE_SYS_FSUID_H
 #ifdef HAVE_SETFSGID
@@ -47,12 +48,24 @@
 #include "gnome-score.h"
 #include "gnome-util.h"
 
+#ifdef G_OS_WIN32
+#include <fcntl.h>
+#include <io.h>
+
+/* Microsoft's strtok() *is* thread-safe, it uses a thread-local
+ * buffer. "Use" the third argument to this macro so gcc doesn't
+ * complain about an unused variable, and we don't have to ifdef out
+ * the definition.
+ */
+#define strtok_r(s, delim, ptrptr) (*(ptrptr) = strtok (s, delim))
+#endif
+
+#include "libgnome-private.h"
+
 #ifndef NSCORES
 #define NSCORES 10
 #endif
 
-#define SCORE_PATH LIBGNOME_LOCALSTATEDIR "/games"
-
 struct command
 {
    gfloat score;
@@ -77,10 +90,13 @@ static gchar *
 gnome_get_score_file_name (const gchar * progname, const gchar * level)
 {
    if (level)
-     return g_strconcat (SCORE_PATH "/",
-			    progname, ".", level, ".scores", NULL);
+     return g_strconcat (LIBGNOME_LOCALSTATEDIR,
+			 G_DIR_SEPARATOR_S "games" G_DIR_SEPARATOR_S,
+			 progname, ".", level, ".scores", NULL);
    else
-     return g_strconcat (SCORE_PATH "/", progname, ".scores", NULL);
+     return g_strconcat (LIBGNOME_LOCALSTATEDIR,
+			 G_DIR_SEPARATOR_S "games" G_DIR_SEPARATOR_S,
+			 progname, ".scores", NULL);
 }
 
 /* This must be wrapped in push_c_locale on the caller */
@@ -129,7 +145,7 @@ log_score (const gchar * progname, const gchar * level, gchar * username,
 
    game_score_file = gnome_get_score_file_name (progname, level);
 
-   infile = fopen (game_score_file, "r");
+   infile = g_fopen (game_score_file, "r");
    if (infile)
      {
        /* make sure we read values from files in a consistent manner */
@@ -164,7 +180,7 @@ log_score (const gchar * progname, const gchar * level, gchar * username,
 
 	fclose (infile);
      }
-   
+
    anitem = g_new(struct ascore_t, 1);
    anitem->score = score;
    anitem->username = g_strdup (username);
@@ -186,7 +202,7 @@ log_score (const gchar * progname, const gchar * level, gchar * username,
 	       break;
 	  }
      }
-   
+
    if (pos < NSCORES)
      {
 	scores = g_list_insert (scores, anitem, pos);
@@ -200,11 +216,15 @@ log_score (const gchar * progname, const gchar * level, gchar * username,
      }
    else
      retval = 0;
-   
+
    /* we dont create the file; it must already exist */
-   truncate (game_score_file, 0);
-   outfile = fopen (game_score_file, "r+");
-   
+   outfile = g_fopen (game_score_file, "r+");
+#ifndef G_OS_WIN32
+   ftruncate (fileno (outfile), 0);
+#else
+   _chsize (fileno (outfile), 0);
+#endif
+
    if (outfile)
      {
 	gnome_i18n_push_c_numeric_locale ();
@@ -216,15 +236,16 @@ log_score (const gchar * progname, const gchar * level, gchar * username,
      perror (game_score_file);
 
    g_free (game_score_file);
-   
+
    g_list_foreach (scores, (GFunc) free_ascore, NULL);
    g_list_free (scores);
-   
+
    return retval;
 }
 
 static int
-gnome_score_child (void)
+gnome_score_child (int infileno,
+		   int outfileno)
 {
    struct command cmd;
    gchar *level;
@@ -232,7 +253,7 @@ gnome_score_child (void)
    gint retval;
 #ifdef HAVE_SETFSGID
    gid_t gid;
-   
+
    gid = getegid ();
    setgid (getgid ());
    setfsgid (gid);
@@ -243,9 +264,9 @@ gnome_score_child (void)
      realname = g_strdup (g_get_user_name ());
    }
 
-   while (read (STDIN_FILENO, &cmd, sizeof cmd) == sizeof(cmd)) {
+   while (read (infileno, &cmd, sizeof cmd) == sizeof(cmd)) {
 	level = g_new (char, cmd.level);
-	if (read (STDIN_FILENO, level, cmd.level) != cmd.level) {
+	if (read (infileno, level, cmd.level) != cmd.level) {
 	  g_free (realname);
 	  return EXIT_FAILURE;
 	}
@@ -255,7 +276,7 @@ gnome_score_child (void)
 	}
 	retval = log_score (defgamename, level, realname, cmd.score,
 			    cmd.ordering);
-	if (write(STDOUT_FILENO, &retval, sizeof retval) != sizeof retval) {
+	if (write(outfileno, &retval, sizeof retval) != sizeof retval) {
 	  g_free (realname);
 	  return EXIT_FAILURE;
 	}
@@ -266,11 +287,26 @@ gnome_score_child (void)
    return EXIT_SUCCESS;
 }
 
-static void 
+#ifdef G_OS_WIN32
+
+static gpointer
+gnome_score_child_thread (gpointer data)
+{
+  int *a = data;
+
+  gnome_score_child (a[0], a[1]);
+
+  return NULL;
+}
+
+#endif
+
+static void
 drop_perms (void)
 {
+#ifndef G_OS_WIN32
    gid_t gid = getegid ();
-   
+
    setregid (getgid (), getgid ());	/* on some os'es (eg linux) this
 					 * incantation will also drop the
 					 * saved gid */
@@ -282,6 +318,7 @@ drop_perms (void)
 		     "get a real OS :)\n");
 	setgid (getgid ());
      }
+#endif
 }
 
 /*********************** external functions **********************************/
@@ -294,7 +331,7 @@ drop_perms (void)
  * in main() if they have been installed setgid to the "games" group. It
  * performs the intialization required to later record information in the
  * scores table and then drops the groups privileges.
- * 
+ *
  * Returns: %0 on success, returns %-1 on failure.
  */
 
@@ -302,7 +339,7 @@ gint
 gnome_score_init (const gchar * gamename)
 {
    int inpipe[2], outpipe[2];
-   
+
    /*
     *creates a child process with which we communicate through a pair of pipes,
     * then drops privileges.
@@ -324,6 +361,7 @@ gnome_score_init (const gchar * gamename)
      }
    outfd = outpipe[1];
    infd = inpipe[0];
+#ifndef G_OS_WIN32
    switch (fork ())
      {
       case 0:
@@ -334,7 +372,7 @@ gnome_score_init (const gchar * gamename)
 	close(inpipe[1]);
 	close(outpipe[0]);
 	close(outpipe[1]);
-	exit (gnome_score_child ());
+	exit (gnome_score_child (STDIN_FILENO, STDOUT_FILENO));
       case -1:
 	close (inpipe[0]);
 	close (inpipe[1]);
@@ -347,6 +385,13 @@ gnome_score_init (const gchar * gamename)
    close(outpipe[0]);
    close(inpipe[1]);
    drop_perms ();
+#else
+   {
+     int a[2] = { outpipe[0], inpipe[1] };
+     g_thread_create (gnome_score_child_thread, a, FALSE, NULL);
+   }
+#endif
+
    return 0;
 }
 
@@ -371,21 +416,23 @@ gnome_score_log (gfloat score,
 {
    struct command cmd;
    gint retval;
-   
+
+#ifndef G_OS_WIN32
    if (getgid () != getegid ())
      {
 	g_error ("gnome_score_init must be called first thing in main()\n");
 	abort ();
      }
+#endif
    if (infd == -1 || outfd == -1)
      return 0;
-   
+
    cmd.score = score;
    if (!level)
      level = "";
    cmd.level = strlen (level) + 1;
    cmd.ordering = higher_to_lower_score_order;
-   
+
    if (write (outfd, &cmd, sizeof cmd) != sizeof(cmd) ||
        write (outfd, level, cmd.level) != cmd.level ||
        read (infd, &retval, sizeof retval) != sizeof(retval))
@@ -424,26 +471,26 @@ gnome_score_get_notable (const gchar * gamename,
    gchar *infile_name;
    FILE *infile;
    gint retval;
-   
+
    g_return_val_if_fail (names != NULL, 0);
    g_return_val_if_fail (scores != NULL, 0);
-   
+
    if (gamename == NULL)
      realname = defgamename;
    else
      realname = gamename;
-   
+
    infile_name = gnome_get_score_file_name (realname, level);
-   
-   infile = fopen (infile_name, "r");
+
+   infile = g_fopen (infile_name, "r");
    g_free (infile_name);
-   
+
    if (infile)
      {
 	*names = g_malloc ((NSCORES + 1) * sizeof (gchar *));
 	*scores = g_malloc ((NSCORES + 1) * sizeof (gfloat));
 	*scoretimes = g_malloc ((NSCORES + 1) * sizeof (time_t));
-	
+
         gnome_i18n_push_c_numeric_locale ();
 
 	for (retval = 0;
